/*-
 * See the file LICENSE for redistribution information.
 *
 * Copyright (c) 2012 Oracle and/or its affiliates.  All rights reserved.
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.  Oracle designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Oracle in the LICENSE file that accompanied this code.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 *
 * $Id$
 */

#include "db_config.h"

#include "db_int.h"
#include "dbinc/db_page.h"
#include "dbinc/log.h"
#include "dbinc/mp.h"
#include "dbinc/txn.h"

/*
 * If the library was built in the 'lite' version, various routines
 * aren't available.  Stub them here, returning an appropriate error.
 */
static int __db_rds __P((ENV *));
static int __db_notransactions __P((ENV *));



/*
 * __dbreg_get_name	- a non-trival 'stub' function!
 *
 * Interface to get name of registered files.  This is mainly diagnostic
 * and the name passed could be transient unless there is something
 * ensuring that the file cannot be closed.
 *
 *	It is used in __lock_stat_print().
 */
int
__dbreg_get_name(env, fid, fnamep, dnamep)
	ENV *env;
	u_int8_t *fid;
	char **fnamep, **dnamep;
{
	*fnamep = "<Some File>";
	*dnamep = NULL;
	COMPQUIET(env, NULL);
	COMPQUIET(fid, NULL);
	return (-1);
}

/*
 * __db_rds --
 *	Error when a restricted distribution invokes a full Berkeley DB feature.
 *
 * PUBLIC: 
 */
static int
__db_rds(env)
	ENV *env;
{
	__db_errx(env, DB_STR("0750",
	    "This RDS build does not include support for this function"));
	__os_stack(env);
	return (DB_OPNOTSUP);
}

/*
 * __db_notransactions --
 *	Error when a Berkeley DB build doesn't include transaction support.
 */
static int
__db_notransactions(env)
	ENV *env;
{
	__db_errx(env, DB_STR("0751",
	    "library build did not include support for transactions"));
	__os_stack(env);
	return (DB_OPNOTSUP);
}

int
__db_check_chksum(env, hdr, db_cipher, chksum, data, data_len, is_hmac)
	ENV *env;
	void *hdr;
	DB_CIPHER *db_cipher;
	u_int8_t *chksum;
	void *data;
	size_t data_len;
	int is_hmac;
{
	COMPQUIET(hdr, NULL);
	COMPQUIET(db_cipher, NULL);
	COMPQUIET(chksum, NULL);
	COMPQUIET(data, NULL);
	COMPQUIET(data_len, 0);
	COMPQUIET(is_hmac, 0);
	return (__db_rds(env));
}

int
__db_relink_log(DB *dbp, DB_TXN *txnp, DB_LSN *ret_lsnp, u_int32_t flags, db_pgno_t pgno, db_pgno_t new_pgno, db_pgno_t prev_pgno, DB_LSN * lsn_prev,
    db_pgno_t next_pgno, DB_LSN * lsn_next)
{
	COMPQUIET(txnp, NULL);
	COMPQUIET(ret_lsnp, NULL);
	COMPQUIET(flags, 0);
	COMPQUIET(pgno, 0);
	COMPQUIET(new_pgno, 0);
	COMPQUIET(prev_pgno, 0);
	COMPQUIET(lsn_prev, NULL);
	COMPQUIET(next_pgno, 0);
	COMPQUIET(lsn_next, NULL);
	return (__db_rds(dbp->env));
}

int
__db_relink_42_print(env, dbtp, lsnp, notused2, info)
	ENV *env;
	DBT *dbtp;
	DB_LSN *lsnp;
	db_recops notused2;
	void *info;
{
	COMPQUIET(dbtp, NULL);
	COMPQUIET(lsnp, NULL);
	COMPQUIET(notused2, 0);
	COMPQUIET(info, NULL);
	return (__db_notransactions(env));
}

int
__db_relink_print(env, dbtp, lsnp, notused2, info)
	ENV *env;
	DBT *dbtp;
	DB_LSN *lsnp;
	db_recops notused2;
	void *info;
{
	COMPQUIET(dbtp, NULL);
	COMPQUIET(lsnp, NULL);
	COMPQUIET(notused2, 0);
	COMPQUIET(info, NULL);
	return (__db_notransactions(env));
}

int
__db_relink_42_recover(env, dbtp, lsnp, op, info)
	ENV *env;
	DBT *dbtp;
	DB_LSN *lsnp;
	db_recops op;
	void *info;
{
	COMPQUIET(dbtp, NULL);
	COMPQUIET(lsnp, NULL);
	COMPQUIET(op, 0);
	COMPQUIET(info, NULL);
	return (__db_notransactions(env));
}

int
__db_relink_recover(env, dbtp, lsnp, op, info)
	ENV *env;
	DBT *dbtp;
	DB_LSN *lsnp;
	db_recops op;
	void *info;
{
	COMPQUIET(dbtp, NULL);
	COMPQUIET(lsnp, NULL);
	COMPQUIET(op, 0);
	COMPQUIET(info, NULL);
	return (__db_notransactions(env));
}

int
__db_upgrade_pp(dbp, fname, flags)
	DB *dbp;
	const char *fname;
	u_int32_t flags;
{
	COMPQUIET(fname, NULL);
	COMPQUIET(flags, 0);
	return (__db_rds(dbp->env));
}

int
__dbreg_log_id(dbp, txn, id, needlock)
	DB *dbp;
	DB_TXN *txn;
	int32_t id;
	int needlock;
{
	COMPQUIET(txn, NULL);
	COMPQUIET(id, 0);
	COMPQUIET(needlock, 0);
	return (__db_notransactions(dbp->env));
}

void
__dbreg_print_fname(env, fnp)
	ENV *env;
	FNAME *fnp;
{
	COMPQUIET(env, NULL);
	COMPQUIET(fnp, NULL);
}

int
__env_get_backup_config(dbenv, config, valuep)
	DB_ENV *dbenv;
	DB_BACKUP_CONFIG config;
	u_int32_t *valuep;
{
	COMPQUIET(config, 0);
	COMPQUIET(valuep, 0);
	return (__db_rds(dbenv->env));
}

int
__env_set_backup_config(dbenv, config, value)
	DB_ENV *dbenv;
	DB_BACKUP_CONFIG config;
	u_int32_t value;
{
	COMPQUIET(config, 0);
	COMPQUIET(value, 0);
	return (__db_rds(dbenv->env));
}

int
__env_get_backup_callbacks(dbenv, openp, writep, closep)
	DB_ENV *dbenv;
	int (**openp)(DB_ENV *, const char *, const char *, void **);
	int (**writep)(DB_ENV *,
		    u_int32_t, u_int32_t, u_int32_t, u_int8_t *, void *);
	int (**closep)(DB_ENV *, const char *, void *);
{
	COMPQUIET(openp, NULL);
	COMPQUIET(writep, NULL);
	COMPQUIET(closep, NULL);
	return (__db_rds(dbenv->env));
}

int
__env_set_backup_callbacks(dbenv, open_func, write_func, close_func)
	DB_ENV *dbenv;
	int (*open_func)(DB_ENV *, const char *, const char *, void **);
	int (*write_func)(DB_ENV *,
	    u_int32_t, u_int32_t, u_int32_t, u_int8_t *, void *);
	int (*close_func)(DB_ENV *, const char *, void *);
{
	COMPQUIET(open_func, NULL);
	COMPQUIET(write_func, NULL);
	COMPQUIET(close_func, NULL);
	return (__db_rds(dbenv->env));
}

int
__env_failchk_pp(dbenv, flags)
	DB_ENV *dbenv;
	u_int32_t flags;
{
	COMPQUIET(flags, 0);
	return (__db_rds(dbenv->env));
}

int
__env_failchk_int(dbenv)
	DB_ENV *dbenv;
{
	return (__db_rds(dbenv->env));
}

size_t
__env_thread_size(env, other_alloc)
	ENV *env;
	size_t other_alloc;
{
	COMPQUIET(env, NULL);
	COMPQUIET(other_alloc, 0);
	return (0);
}

size_t
__env_thread_max(env)
	ENV *env;
{
	COMPQUIET(env, NULL);
	return (0);
}

int
__env_thread_init(env, during_creation)
	ENV *env;
	int during_creation;
{
	COMPQUIET(env, NULL);
	COMPQUIET(during_creation, 0);
	return (0);
}

void
__env_thread_destroy(env)
	ENV *env;
{
	COMPQUIET(env, NULL);
}

int
__env_set_state(env, ipp, state)
	ENV *env;
	DB_THREAD_INFO **ipp;
	DB_THREAD_STATE state;
{
	COMPQUIET(ipp, NULL);
	COMPQUIET(state, 0);
	return (__db_rds(env));
}

int
__env_init_rec(env, version)
	ENV *env;
	u_int32_t version;
{
	COMPQUIET(version, 0);
	return (__db_rds(env));
}

int
__env_init_rec_42(env)
	ENV *env;
{
	return (__db_rds(env));
}

int
__env_init_rec_43(env)
	ENV *env;
{
	return (__db_rds(env));
}

int
__env_init_rec_46(env)
	ENV *env;
{
	return (__db_rds(env));
}

int
__env_init_rec_47(env)
	ENV *env;
{
	return (__db_rds(env));
}

int
__env_init_rec_48(env)
	ENV *env;
{
	return (__db_rds(env));
}

int
__envreg_register(env, need_recoveryp, flags)
	ENV *env;
	int *need_recoveryp;
	u_int32_t flags;
{
	COMPQUIET(need_recoveryp, NULL);
	COMPQUIET(flags, 0);
	return (__db_rds(env));
}

int
__envreg_unregister(env, recovery_failed)
	ENV *env;
	int recovery_failed;
{
	COMPQUIET(recovery_failed, 0);
	return (__db_rds(env));
}

int
__envreg_xunlock(env)
	ENV *env;
{
	return (__db_rds(env));
}

int
__envreg_isalive(dbenv, pid, tid, flags )
	DB_ENV *dbenv;
	pid_t pid;
	db_threadid_t tid;
	u_int32_t flags;
{
	COMPQUIET(pid, 0);
	DB_THREADID_INIT(tid);
	COMPQUIET(flags, 0);
	return (__db_rds(dbenv->env));
}

int
__log_open(env)
	ENV *env;
{
	return (__db_notransactions(env));
}

size_t
__log_region_size(env)
	ENV *env;
{
	COMPQUIET(env, NULL);
	return (0);
}

size_t
__log_region_max(env)
	ENV *env;
{
	COMPQUIET(env, NULL);
	return (0);
}

u_int32_t
__log_region_mutex_count(env)
	ENV *env;
{
	COMPQUIET(env, NULL);
	return (0);
}

u_int32_t
__log_region_mutex_max(env)
	ENV *env;
{
	COMPQUIET(env, NULL);
	return (0);
}

int
__log_archive_pp(dbenv, listp, flags)
	DB_ENV *dbenv;
	char ***listp;
	u_int32_t flags;
{
	COMPQUIET(listp, NULL);
	COMPQUIET(flags, 0);
	return (__db_notransactions(dbenv->env));
}

int
__log_archive(env, listp, flags)
	ENV *env;
	char ***listp;
	u_int32_t flags;
{
	COMPQUIET(listp, NULL);
	COMPQUIET(flags, 0);
	return (__db_notransactions(env));
}

int
__log_env_create(dbenv)
	DB_ENV *dbenv;
{
	COMPQUIET(dbenv, NULL);
	return (0);
}

void
__log_env_destroy(dbenv)
	DB_ENV *dbenv;
{
	COMPQUIET(dbenv, NULL);
}

int
__log_get_config(dbenv, which, onp)
	DB_ENV *dbenv;
	u_int32_t which;
	int *onp;
{
	COMPQUIET(which, 0);
	COMPQUIET(onp, NULL);
	return (__db_notransactions(dbenv->env));
}

int
__log_set_lg_bsize(dbenv, lg_bsize)
	DB_ENV *dbenv;
	u_int32_t lg_bsize;
{
	COMPQUIET(lg_bsize, 0);
	return (__db_notransactions(dbenv->env));
}

int
__log_set_lg_filemode(dbenv, lg_mode)
	DB_ENV *dbenv;
	int lg_mode;
{
	COMPQUIET(lg_mode, 0);
	return (__db_notransactions(dbenv->env));
}

int
__log_set_lg_max(dbenv, lg_max)
	DB_ENV *dbenv;
	u_int32_t lg_max;
{
	COMPQUIET(lg_max, 0);
	return (__db_notransactions(dbenv->env));
}

int
__log_set_lg_regionmax(dbenv, lg_regionmax)
	DB_ENV *dbenv;
	u_int32_t lg_regionmax;
{
	COMPQUIET(lg_regionmax, 0);
	return (__db_notransactions(dbenv->env));
}

int
__log_set_lg_dir(dbenv, dir)
	DB_ENV *dbenv;
	const char *dir;
{
	COMPQUIET(dir, NULL);
	return (__db_notransactions(dbenv->env));
}

int
__log_set_config(dbenv, flags, on)
	DB_ENV *dbenv;
	u_int32_t flags;
	int on;
{
	COMPQUIET(flags, 0);
	COMPQUIET(on, 0);
	return (__db_notransactions(dbenv->env));
}

int
__log_set_config_int(dbenv, flags, on, in_open)
	DB_ENV *dbenv;
	u_int32_t flags;
	int on;
	int in_open;
{
	COMPQUIET(flags, 0);
	COMPQUIET(on, 0);
	COMPQUIET(in_open, 0);
	return (__db_notransactions(dbenv->env));
}

int
__log_flush_commit(env, lsnp, flags)
	ENV *env;
	const DB_LSN *lsnp;
	u_int32_t flags;
{
	COMPQUIET(lsnp, NULL);
	COMPQUIET(flags, 0);
	return (__db_notransactions(env));
}

int
__log_flush_pp(dbenv, lsn)
	DB_ENV *dbenv;
	const DB_LSN *lsn;
{
	COMPQUIET(lsn, NULL);
	return (__db_notransactions(dbenv->env));
}

int
__log_flush(env, lsn)
	ENV *env;
	const DB_LSN *lsn;
{
	COMPQUIET(lsn, NULL);
	return (__db_notransactions(env));
}

int
__log_flush_int(dblp, lsnp, release)
	DB_LOG *dblp;
	const DB_LSN *lsnp;
	int release;
{
	COMPQUIET(lsnp, NULL);
	COMPQUIET(release, 0);
	return (__db_notransactions(dblp->env));
}

int
__log_stat_print(env, flags)
	ENV *env;
	u_int32_t flags;
{
	COMPQUIET(flags, 0);
	return (__db_notransactions(env));
}


int
log_compare(lsn0, lsn1)
	const DB_LSN *lsn0, *lsn1;
{
	COMPQUIET(lsn0, NULL);
	COMPQUIET(lsn1, NULL);
	return (__db_notransactions(NULL));
}

int
__log_check_page_lsn(env, dbp, lsnp)
	ENV *env;
	DB *dbp;
	DB_LSN *lsnp;
{
	COMPQUIET(dbp, NULL);
	COMPQUIET(lsnp, NULL);
	return (__db_notransactions(env));
}

int
__log_current_lsn_int(env, lsnp, mbytesp, bytesp)
	ENV *env;
	DB_LSN *lsnp;
	u_int32_t *mbytesp, *bytesp;
{
	COMPQUIET(lsnp, NULL);
	COMPQUIET(mbytesp, NULL);
	COMPQUIET(bytesp, NULL);
	return (__db_notransactions(env));
}

int
__log_env_refresh(env)
	ENV *env;
{
	return (__db_notransactions(env));
}

#ifdef STDC_HEADERS
int
__log_put_record(ENV *env, DB *dbp, DB_TXN *txnp, DB_LSN *ret_lsnp,
    u_int32_t flags, u_int32_t rectype, u_int32_t has_data, u_int32_t size,
    DB_LOG_RECSPEC *spec, ...)
#else
int
__log_put_record(env, dbp, txnp, ret_lsnp,
    flags, rectype, has_data, size, spec, va_alist);
	ENV *env;
	DB *dbp;
	DB_TXN *txnp;
	DB_LSN *ret_lsnp;
	u_int32_t flags;
	u_int32_t rectype;
	u_int32_t has_data;
	u_int32_t size;
	DB_LOG_RECSPEC *spec;
	va_dcl
#endif
{
	COMPQUIET(dbp, NULL);
	COMPQUIET(txnp, NULL);
	COMPQUIET(ret_lsnp, NULL);
	COMPQUIET(flags, 0);
	COMPQUIET(rectype, 0);
	COMPQUIET(has_data, 0);
	COMPQUIET(size, 0);
	COMPQUIET(spec, NULL);
	return (__db_notransactions(env));
}

int
__log_read_record(env, dbpp, td, recbuf, spec, size, argpp)
	ENV *env;
	DB **dbpp;
	void *td;
	void *recbuf;
	DB_LOG_RECSPEC *spec;
	u_int32_t size;
	void **argpp;
{
	COMPQUIET(dbpp, NULL);
	COMPQUIET(td, NULL);
	COMPQUIET(recbuf, NULL);
	COMPQUIET(spec, NULL);
	COMPQUIET(size, 0);
	COMPQUIET(argpp, NULL);
	return (__db_notransactions(env));
}

int
__db_relink_42_verify(env, dbtp, lsnp, notused2, lvhp)
	ENV *env;
	DBT *dbtp;
	DB_LSN *lsnp;
	db_recops notused2;
	void *lvhp;
{
	COMPQUIET(dbtp, NULL);
	COMPQUIET(lsnp, NULL);
	COMPQUIET(notused2, 0);
	COMPQUIET(lvhp, NULL);
	return (__db_notransactions(env));
}

int
__db_relink_verify(env, dbtp, lsnp, notused2, lvhp)
	ENV *env;
	DBT *dbtp;
	DB_LSN *lsnp;
	db_recops notused2;
	void *lvhp;
{
	COMPQUIET(dbtp, NULL);
	COMPQUIET(lsnp, NULL);
	COMPQUIET(notused2, 0);
	COMPQUIET(lvhp, NULL);
	return (__db_notransactions(env));
}

int
__db_dbtxn_remove(dbp, ip, txn, name, subdb)
	DB *dbp;
	DB_THREAD_INFO *ip;
	DB_TXN *txn;
	const char *name, *subdb;
{
	COMPQUIET(ip, NULL);
	COMPQUIET(txn, NULL);
	COMPQUIET(name, NULL);
	COMPQUIET(subdb, NULL);

	return (__db_notransactions(dbp->env));
}

void
__db_chksum(hdr, data, data_len, mac_key, store)
	void *hdr;
	u_int8_t *data;
	size_t data_len;
	u_int8_t *mac_key;
	u_int8_t *store;
{
	COMPQUIET(hdr, NULL);
	COMPQUIET(data, NULL);
	COMPQUIET(data_len, 0);
	COMPQUIET(mac_key, NULL);
	COMPQUIET(store, NULL);
	(void)__db_rds(NULL);
}

int
__fop_subdb_setup(dbp, ip, txn, mname, name, mode, flags)
	DB *dbp;
	DB_THREAD_INFO *ip;
	DB_TXN *txn;
	const char *mname, *name;
	int mode;
	u_int32_t flags;
{
	COMPQUIET(dbp, NULL);
	COMPQUIET(ip, NULL);
	COMPQUIET(mname, NULL);
	COMPQUIET(name, NULL);
	COMPQUIET(mode, 0);
	COMPQUIET(flags, 0);
	return (__db_rds(NULL));
}

int
__fop_dummy(dbp, txn, old, new)
	DB *dbp;
	DB_TXN *txn;
	const char *old, *new;
{
	COMPQUIET(txn, NULL);
	COMPQUIET(old, NULL);
	COMPQUIET(new, NULL);
	return (__db_notransactions(dbp->env));
}

int
__memp_backup_open(env, mpf, dbfile, target, flags, fpp, handlep)
	ENV *env;
	DB_MPOOLFILE *mpf;
	const char *dbfile;
	const char *target;
	u_int32_t flags;
	DB_FH **fpp;
	void **handlep;
{
	COMPQUIET(mpf, NULL);
	COMPQUIET(dbfile, NULL);
	COMPQUIET(target, NULL);
	COMPQUIET(flags, 0);
	COMPQUIET(fpp, NULL);
	COMPQUIET(handlep, NULL);
	return (__db_rds(env));
}

int
__memp_backup_mpf(env, mpf, ip, first_pgno, last_pgno, fp, handle, flags)
	ENV *env;
	DB_MPOOLFILE *mpf;
	DB_THREAD_INFO *ip;
	db_pgno_t first_pgno, last_pgno;
	DB_FH *fp;
	void *handle;
	u_int32_t flags;
{
	COMPQUIET(mpf, NULL);
	COMPQUIET(ip, NULL);
	COMPQUIET(first_pgno, 0);
	COMPQUIET(last_pgno, 0);
	COMPQUIET(fp, NULL);
	COMPQUIET(handle, NULL);
	COMPQUIET(flags, 0);
	return (__db_rds(env));
}

int
__memp_backup_close(env, mpf, dbfile, fp, handle)
	ENV *env;
	DB_MPOOLFILE *mpf;
	const char *dbfile;
	DB_FH *fp;
	void *handle;
{
	COMPQUIET(mpf, NULL);
	COMPQUIET(dbfile, NULL);
	COMPQUIET(fp, NULL);
	COMPQUIET(handle, NULL);
	return (__db_rds(env));
}

int
__memp_bh_freeze(dbmp, infop, hp, bhp, need_frozenp)
	DB_MPOOL *dbmp;
	REGINFO *infop;
	DB_MPOOL_HASH *hp;
	BH *bhp;
	int *need_frozenp;
{
	COMPQUIET(infop, NULL);
	COMPQUIET(hp, NULL);
	COMPQUIET(bhp, NULL);
	COMPQUIET(need_frozenp, NULL);
	return (__db_rds(dbmp->env));
}

int
__memp_bh_thaw(dbmp, infop, hp, frozen_bhp, alloc_bhp)
	DB_MPOOL *dbmp;
	REGINFO *infop;
	DB_MPOOL_HASH *hp;
	BH *frozen_bhp, *alloc_bhp;
{
	COMPQUIET(infop, NULL);
	COMPQUIET(hp, NULL);
	COMPQUIET(frozen_bhp, NULL);
	COMPQUIET(alloc_bhp, NULL);
	return (__db_rds(dbmp->env));
}

int
__memp_bh_settxn(dbmp, mfp, bhp, vtd)
	DB_MPOOL *dbmp;
	MPOOLFILE *mfp;
	BH *bhp;
	void *vtd;
{
	COMPQUIET(mfp, NULL);
	COMPQUIET(bhp, NULL);
	COMPQUIET(vtd, NULL);
	return (__db_notransactions(dbmp->env));
}

int
__memp_skip_curadj(dbc, pgno)
	DBC *dbc;
	db_pgno_t pgno;
{
	COMPQUIET(dbc, NULL);
	COMPQUIET(pgno, 0);
	return (FALSE);
}

int
__txn_checkpoint_pp(dbenv, kbytes, minutes, flags)
	DB_ENV *dbenv;
	u_int32_t kbytes, minutes, flags;
{
	COMPQUIET(kbytes, 0);
	COMPQUIET(minutes, 0);
	COMPQUIET(flags, 0);
	return (__db_notransactions(dbenv->env));
}

int
__txn_checkpoint(env, kbytes, minutes, flags)
	ENV *env;
	u_int32_t kbytes, minutes, flags;
{
	COMPQUIET(kbytes, 0);
	COMPQUIET(minutes, 0);
	COMPQUIET(flags, 0);
	return (__db_notransactions(env));
}

int
__txn_env_create(dbenv)
	DB_ENV *dbenv;
{
	COMPQUIET(dbenv, NULL);
	return (0);
}

void
__txn_env_destroy(dbenv)
	DB_ENV *dbenv;
{
	COMPQUIET(dbenv, NULL);
}

int
__txn_set_timeout(txn, timeout, op)
	DB_TXN *txn;
	db_timeout_t timeout;
	u_int32_t op;
{
	COMPQUIET(txn, NULL);
	COMPQUIET(timeout, 0);
	COMPQUIET(op, 0);
	return (0);
}

int
__txn_set_tx_max(dbenv, tx_max)
	DB_ENV *dbenv;
	u_int32_t tx_max;
{
	COMPQUIET(tx_max, 0);
	return (__db_notransactions(dbenv->env));
}

int
__txn_set_tx_timestamp(dbenv, timestamp)
	DB_ENV *dbenv;
	time_t *timestamp;
{
	COMPQUIET(timestamp, NULL);
	return (__db_notransactions(dbenv->env));
}

int
__txn_begin(env, ip, parent, txnpp, flags)
	ENV *env;
	DB_THREAD_INFO *ip;
	DB_TXN *parent, **txnpp;
	u_int32_t flags;
{
	COMPQUIET(ip, NULL);
	COMPQUIET(parent, NULL);
	COMPQUIET(txnpp, NULL);
	COMPQUIET(flags, 0);
	return (__db_notransactions(env));
}

int
__txn_commit(txn, flags)
	DB_TXN *txn;
	u_int32_t flags;
{
	COMPQUIET(txn, NULL);
	COMPQUIET(flags, 0);
	return (__db_notransactions(NULL));
	
}

int
__txn_abort(txn)
	DB_TXN *txn;
{
	COMPQUIET(txn, NULL);
	return (__db_notransactions(NULL));
}

int
__txn_preclose(env)
	ENV *env;
{
	return (__db_notransactions(env));
}

int
__dbreg_setup(dbp, fname, dname, create_txnid)
	DB *dbp;
	const char *fname, *dname;
	u_int32_t create_txnid;
{
	COMPQUIET(fname, NULL);
	COMPQUIET(dname, NULL);
	COMPQUIET(create_txnid, 0);

	return (__db_notransactions(dbp->env));
}

int
__dbreg_new_id(dbp, txn)
	DB *dbp;
	DB_TXN *txn;
{
	COMPQUIET(txn, NULL);
	return (__db_notransactions(dbp->env));
}

int
__dbreg_revoke_id(dbp, have_lock, force_id)
	DB *dbp;
	int have_lock;
	int32_t force_id;
{
	COMPQUIET(have_lock, 0);
	COMPQUIET(force_id, 0);
	return (__db_notransactions(dbp->env));
}

int
__dbreg_teardown(dbp)
	DB *dbp;
{
	return (__db_notransactions(dbp->env));
}

int
__dbreg_close_id(dbp, txn, op)
	DB *dbp;
	DB_TXN *txn;
	u_int32_t op;
{
	COMPQUIET(txn, NULL);
	COMPQUIET(op, 0);
	return (__db_notransactions(dbp->env));
}

int
__dbreg_stat_print(env, flags)
	ENV *env;
	u_int32_t flags;
{
	COMPQUIET(flags, 0);
	return (__db_notransactions(env));
}

/* txn_util.c */
int
__txn_closeevent(env, txn, dbp)
	ENV *env;
	DB_TXN *txn;
	DB *dbp;
{
	COMPQUIET(txn, NULL);
	COMPQUIET(dbp, NULL);
	return (__db_notransactions(env));
}

int
__txn_remevent(env, txn, name, fileid, inmem)
	ENV *env;
	DB_TXN *txn;
	const char *name;
	u_int8_t *fileid;
	int inmem;
{
	COMPQUIET(txn, NULL);
	COMPQUIET(name, NULL);
	COMPQUIET(fileid, NULL);
	COMPQUIET(inmem, 0);
	return (__db_notransactions(env));
}

void
__txn_remrem(env, txn, name)
	ENV *env;
	DB_TXN *txn;
	const char *name;
{
	COMPQUIET(txn, NULL);
	COMPQUIET(name, NULL);
	(void) __db_notransactions(env);
}

int
__txn_lockevent(env, txn, dbp, lock, locker)
	ENV *env;
	DB_TXN *txn;
	DB *dbp;
	DB_LOCK *lock;
	DB_LOCKER *locker;
{
	COMPQUIET(txn, NULL);
	COMPQUIET(dbp, NULL);
	COMPQUIET(lock, NULL);
	COMPQUIET(locker, NULL);
	return (__db_notransactions(env));
}

void
__txn_remlock(env, txn, lock, locker)
	ENV *env;
	DB_TXN *txn;
	DB_LOCK *lock;
	DB_LOCKER *locker;
{
	COMPQUIET(txn, NULL);
	COMPQUIET(lock, NULL);
	COMPQUIET(locker, NULL);
	(void) __db_notransactions(env);
}

int
__txn_record_fname(env, txn, fname)
	ENV *env;
	DB_TXN *txn;
	FNAME *fname;
{
	COMPQUIET(txn, NULL);
	COMPQUIET(fname, NULL);
	return (__db_notransactions(env));
}

void
__txn_remove_fe_watermark(txn, db)
     DB_TXN *txn;
     DB *db;
{
	COMPQUIET(txn, NULL);
	COMPQUIET(db, NULL);
}

void
__txn_add_fe_watermark(txn, db, pgno)
     DB_TXN *txn;
     DB *db;
     db_pgno_t pgno;
{
	COMPQUIET(txn, NULL);
	COMPQUIET(db, NULL);
	COMPQUIET(pgno, 0);
    	return;
}

int
__txn_pg_above_fe_watermark(txn, mpf, pgno)
     DB_TXN *txn;
     MPOOLFILE *mpf;
     db_pgno_t pgno;
{
	COMPQUIET(txn, NULL);
	COMPQUIET(mpf, NULL);
	COMPQUIET(pgno, 0);
	return (0);
}

int
__txn_stat_print(env, flags)
	ENV *env;
	u_int32_t flags;
{
	COMPQUIET(flags, 0);
	return (__db_notransactions(env));
}

/* txn_region.c */
int
__txn_open(env)
	ENV *env;
{
	return (__db_notransactions(env));
}

int
__txn_env_refresh(env)
	ENV *env;
{
	return (__db_notransactions(env));
}

int
__txn_id_set(env, cur_txnid, max_txnid)
	ENV *env;
	u_int32_t cur_txnid, max_txnid;
{
	COMPQUIET(env, NULL);
	COMPQUIET(cur_txnid, 0);
	COMPQUIET(max_txnid, 0);
	return (0);
}

size_t
__txn_region_max(env)
	ENV *env;
{
	COMPQUIET(env, NULL);
	return (0);
}

size_t
__txn_region_size(env)
	ENV *env;
{
	COMPQUIET(env, NULL);
	return (0);
}

int
__txn_oldest_reader(env, lsnp)
	ENV *env;
	DB_LSN *lsnp;
{
	COMPQUIET(lsnp, NULL);
	return (__db_notransactions(env));
}

int
__txn_add_buffer(env, td)
	ENV *env;
	TXN_DETAIL *td;
{
	COMPQUIET(td, NULL);
	return (__db_notransactions(env));
}

int
__txn_remove_buffer(env, td, hash_mtx)
	ENV *env;
	TXN_DETAIL *td;
	db_mutex_t hash_mtx;
{
	COMPQUIET(td, NULL);
	COMPQUIET(hash_mtx, 0);
	return (__db_notransactions(env));
}

u_int32_t
__txn_region_mutex_count(env)
	ENV *env;
{
	COMPQUIET(env, NULL);
	/*
	 * We need  a mutex for DB_TXNMGR structure, two mutexes for
	 * the DB_TXNREGION structure.
	 */
	return (1 + 2);
}

u_int32_t
__txn_region_mutex_max(env)
	ENV *env;
{
	COMPQUIET(env, NULL);
	return (0);
}

int
__cdsgroup_begin_pp(dbenv, txnpp)
	DB_ENV *dbenv;
	DB_TXN **txnpp;
{
	COMPQUIET(txnpp, NULL);
	return (__db_notransactions(dbenv->env));
}

int
__cdsgroup_begin(env, txnpp)
	ENV *env;
	DB_TXN **txnpp;
{
	COMPQUIET(txnpp, NULL);
	return (__db_notransactions(env));
}

int
__db_check_txn(dbp, txn, assoc_locker, read_op)
	DB *dbp;
	DB_TXN *txn;
	DB_LOCKER *assoc_locker;
	int read_op;
{
	COMPQUIET(dbp, NULL);
	COMPQUIET(txn, NULL);
	COMPQUIET(assoc_locker, NULL);
	COMPQUIET(read_op, 0);
	return (0);
}

int
__db_join_close(dbc)
	DBC *dbc;
{
	return (__db_rds(dbc->env));
}

DB_LOG_RECSPEC __bam_split_desc[] = { { 0 , 0, NULL,  { '\0' } } };
DB_LOG_RECSPEC __bam_split_48_desc[] = {  { 0 , 0, NULL,  { '\0' } } };
DB_LOG_RECSPEC __bam_split_42_desc[] = { { 0 , 0, NULL,  { '\0' } } };
DB_LOG_RECSPEC __bam_rsplit_desc[] = { { 0 , 0, NULL,  { '\0' } } };
DB_LOG_RECSPEC __bam_adj_desc[] = { { 0 , 0, NULL,  { '\0' } } };
DB_LOG_RECSPEC __bam_cadjust_desc[] = { { 0 , 0, NULL,  { '\0' } } };
DB_LOG_RECSPEC __bam_cdel_desc[] = { { 0 , 0, NULL,  { '\0' } } };
DB_LOG_RECSPEC __bam_repl_desc[] = { { 0 , 0, NULL,  { '\0' } } };
DB_LOG_RECSPEC __bam_irep_desc[] = { { 0 , 0, NULL,  { '\0' } } };
DB_LOG_RECSPEC __bam_root_desc[] = { { 0 , 0, NULL,  { '\0' } } };
DB_LOG_RECSPEC __bam_curadj_desc[] = { { 0 , 0, NULL,  { '\0' } } };
DB_LOG_RECSPEC __bam_rcuradj_desc[] = { { 0 , 0, NULL,  { '\0' } } };
DB_LOG_RECSPEC __bam_relink_43_desc[] = { { 0 , 0, NULL,  { '\0' } } };
DB_LOG_RECSPEC __bam_merge_44_desc[] = { { 0 , 0, NULL,  { '\0' } } };
DB_LOG_RECSPEC __crdel_metasub_desc[] = { { 0 , 0, NULL,  { '\0' } } };
DB_LOG_RECSPEC __crdel_inmem_create_desc[] = { { 0 , 0, NULL,  { '\0' } } };
DB_LOG_RECSPEC __crdel_inmem_rename_desc[] = { { 0 , 0, NULL,  { '\0' } } };
DB_LOG_RECSPEC __crdel_inmem_remove_desc[] = { { 0 , 0, NULL,  { '\0' } } };
DB_LOG_RECSPEC __db_addrem_desc[] = { { 0 , 0, NULL,  { '\0' } } };
DB_LOG_RECSPEC __db_addrem_42_desc[] = { { 0 , 0, NULL,  { '\0' } } };
DB_LOG_RECSPEC __db_big_desc[] = { { 0 , 0, NULL,  { '\0' } } };
DB_LOG_RECSPEC __db_big_42_desc[] = { { 0 , 0, NULL,  { '\0' } } };
DB_LOG_RECSPEC __db_ovref_desc[] = { { 0 , 0, NULL,  { '\0' } } };
DB_LOG_RECSPEC __db_relink_42_desc[] = { { 0 , 0, NULL,  { '\0' } } };
DB_LOG_RECSPEC __db_debug_desc[] = { { 0 , 0, NULL,  { '\0' } } };
DB_LOG_RECSPEC __db_noop_desc[] = { { 0 , 0, NULL,  { '\0' } } };
DB_LOG_RECSPEC __db_pg_alloc_42_desc[] = { { 0 , 0, NULL,  { '\0' } } };
DB_LOG_RECSPEC __db_pg_alloc_desc[] = { { 0 , 0, NULL,  { '\0' } } };
DB_LOG_RECSPEC __db_pg_free_42_desc[] = { { 0 , 0, NULL,  { '\0' } } };
DB_LOG_RECSPEC __db_pg_free_desc[] = { { 0 , 0, NULL,  { '\0' } } };
DB_LOG_RECSPEC __db_cksum_desc[] = { { 0 , 0, NULL,  { '\0' } } };
DB_LOG_RECSPEC __db_pg_freedata_42_desc[] = { { 0 , 0, NULL,  { '\0' } } };
DB_LOG_RECSPEC __db_pg_freedata_desc[] = { { 0 , 0, NULL,  { '\0' } } };
DB_LOG_RECSPEC __db_pg_init_desc[] = { { 0 , 0, NULL,  { '\0' } } };
DB_LOG_RECSPEC __db_pg_sort_44_desc[] = { { 0 , 0, NULL,  { '\0' } } };
DB_LOG_RECSPEC __db_pg_trunc_desc[] = { { 0 , 0, NULL,  { '\0' } } };
DB_LOG_RECSPEC __db_realloc_desc[] = { { 0 , 0, NULL,  { '\0' } } };
DB_LOG_RECSPEC __db_relink_desc[] = { { 0 , 0, NULL,  { '\0' } } };
DB_LOG_RECSPEC __db_merge_desc[] = { { 0 , 0, NULL,  { '\0' } } };
DB_LOG_RECSPEC __db_pgno_desc[] = { { 0 , 0, NULL,  { '\0' } } };
DB_LOG_RECSPEC __dbreg_register_desc[] = { { 0 , 0, NULL,  { '\0' } } };
DB_LOG_RECSPEC __fop_create_42_desc[] = { { 0 , 0, NULL,  { '\0' } } };
DB_LOG_RECSPEC __fop_create_desc[] = { { 0 , 0, NULL,  { '\0' } } };
DB_LOG_RECSPEC __fop_remove_desc[] = { { 0 , 0, NULL,  { '\0' } } };
DB_LOG_RECSPEC __fop_write_42_desc[] = { { 0 , 0, NULL,  { '\0' } } };
DB_LOG_RECSPEC __fop_write_desc[] = { { 0 , 0, NULL,  { '\0' } } };
DB_LOG_RECSPEC __fop_rename_42_desc[] = { { 0 , 0, NULL,  { '\0' } } };
DB_LOG_RECSPEC __fop_rename_noundo_46_desc[] = { { 0 , 0, NULL,  { '\0' } } };
DB_LOG_RECSPEC __fop_rename_desc[] = { { 0 , 0, NULL,  { '\0' } } };
DB_LOG_RECSPEC __fop_rename_noundo_desc[] = { { 0 , 0, NULL,  { '\0' } } };
DB_LOG_RECSPEC __fop_file_remove_desc[] = { { 0 , 0, NULL,  { '\0' } } };
DB_LOG_RECSPEC __ham_insdel_desc[] = { { 0 , 0, NULL,  { '\0' } } };
DB_LOG_RECSPEC __ham_insdel_42_desc[] = { { 0 , 0, NULL,  { '\0' } } };
DB_LOG_RECSPEC __ham_newpage_desc[] = { { 0 , 0, NULL,  { '\0' } } };
DB_LOG_RECSPEC __ham_splitdata_desc[] = { { 0 , 0, NULL,  { '\0' } } };
DB_LOG_RECSPEC __ham_replace_desc[] = { { 0 , 0, NULL,  { '\0' } } };
DB_LOG_RECSPEC __ham_replace_42_desc[] = { { 0 , 0, NULL,  { '\0' } } };
DB_LOG_RECSPEC __ham_copypage_desc[] = { { 0 , 0, NULL,  { '\0' } } };
DB_LOG_RECSPEC __ham_metagroup_42_desc[] = { { 0 , 0, NULL,  { '\0' } } };
DB_LOG_RECSPEC __ham_metagroup_desc[] = { { 0 , 0, NULL,  { '\0' } } };
DB_LOG_RECSPEC __ham_groupalloc_42_desc[] = { { 0 , 0, NULL,  { '\0' } } };
DB_LOG_RECSPEC __ham_groupalloc_desc[] = { { 0 , 0, NULL,  { '\0' } } };
DB_LOG_RECSPEC __ham_changeslot_desc[] = { { 0 , 0, NULL,  { '\0' } } };
DB_LOG_RECSPEC __ham_contract_desc[] = { { 0 , 0, NULL,  { '\0' } } };
DB_LOG_RECSPEC __ham_curadj_desc[] = { { 0 , 0, NULL,  { '\0' } } };
DB_LOG_RECSPEC __ham_chgpg_desc[] = { { 0 , 0, NULL,  { '\0' } } };
DB_LOG_RECSPEC __heap_addrem_desc[] = { { 0 , 0, NULL,  { '\0' } } };
DB_LOG_RECSPEC __heap_addrem_50_desc[] = { { 0 , 0, NULL,  { '\0' } } };
DB_LOG_RECSPEC __heap_pg_alloc_desc[] = { { 0 , 0, NULL,  { '\0' } } };
DB_LOG_RECSPEC __heap_trunc_meta_desc[] = { { 0 , 0, NULL,  { '\0' } } };
DB_LOG_RECSPEC __heap_trunc_page_desc[] = { { 0 , 0, NULL,  { '\0' } } };
DB_LOG_RECSPEC __qam_incfirst_desc[] = { { 0 , 0, NULL,  { '\0' } } };
DB_LOG_RECSPEC __qam_mvptr_desc[] = { { 0 , 0, NULL,  { '\0' } } };
DB_LOG_RECSPEC __qam_del_desc[] = { { 0 , 0, NULL,  { '\0' } } };
DB_LOG_RECSPEC __qam_add_desc[] = { { 0 , 0, NULL,  { '\0' } } };
DB_LOG_RECSPEC __qam_delext_desc[] = { { 0 , 0, NULL,  { '\0' } } };
DB_LOG_RECSPEC __repmgr_member_desc[] = { { 0 , 0, NULL,  { '\0' } } };
DB_LOG_RECSPEC __txn_regop_42_desc[] = { { 0 , 0, NULL,  { '\0' } } };
DB_LOG_RECSPEC __txn_regop_desc[] = { { 0 , 0, NULL,  { '\0' } } };
DB_LOG_RECSPEC __txn_ckp_42_desc[] = { { 0 , 0, NULL,  { '\0' } } };
DB_LOG_RECSPEC __txn_ckp_desc[] = { { 0 , 0, NULL,  { '\0' } } };
DB_LOG_RECSPEC __txn_child_desc[] = { { 0 , 0, NULL,  { '\0' } } };
DB_LOG_RECSPEC __txn_xa_regop_42_desc[] = { { 0 , 0, NULL,  { '\0' } } };
DB_LOG_RECSPEC __txn_prepare_desc[] = { { 0 , 0, NULL,  { '\0' } } };
DB_LOG_RECSPEC __txn_recycle_desc[] = { { 0 , 0, NULL,  { '\0' } } };
